#!/usr/bin/ksh

function print_usage {
   print "usage: migbkup <install location> <backup directory> jndi"
   print "\n<install location> is the path to the current LDAP install, i.e. /usr/ldap"
   print "<backup directory> is the directory where the files will be copied"
   print "jndi will backup IBM JNDI specific files, use for 3.2.2 only"
   print "\nThis script will copy the following files from the <install location>/etc directory:"
   print "slapd32.conf/ibmslapd.conf"
   print "V3.ibm.at"
   print "V3.ibm.oc"
   print "V3.system.at"
   print "V3.system.oc"
   print "V3.user.at"
   print "V3.user.oc"
   print "V3.modifiedschema"
   print "\nIf jndi is specified, the <install location>/jre/bin and"
   print "<install location>/jre/lib directories will be copied, as"
   print "well as the following files from <install location>/java:"
   print "Ibmjcefw.jar"
   print "Ibmjceprovider.jar"
   print "IBMjgssprovider.jar"
   print "Local_policy.jar"
   print "US_export_policy.jar"
   print "Krb5.ini"
   print "Ibmjndi.jar"
   print "Ibmjndi.zip"
   exit
}

function cfg_schema_files {
   print "Backing up schema and configuration files ....."
   set -A sfiles
   sfiles[0]="V3.ibm.at"
   sfiles[1]="V3.ibm.oc"
   sfiles[2]="V3.system.at"
   sfiles[3]="V3.system.oc"
   sfiles[4]="V3.user.at"
   sfiles[5]="V3.user.oc"
   sfiles[6]="V3.modifiedschema"
   mkdir $bkdir/etc > /dev/null 2>&1
   for filename in ${sfiles[*]} ;do
      if [ -f $instdir/etc/$filename ]
      then 
         cp $instdir/etc/$filename $bkdir/etc > /dev/null 2>&1
         print "Copying: file" $instdir/etc/$filename "."
      else
         print "Error: file" $instdir/etc/$filename "does not exist and is needed for successful migration."
      fi
   done
   set -A cfiles
   cfiles[0]="ibmslapd.conf"
   cfiles[1]="slapd32.conf"
   found="false"
   for filename in ${cfiles[*]} ;do
      if [ -f $instdir/etc/$filename ]
      then 
         if [ $found = "false" ]
         then
            cp $instdir/etc/$filename $bkdir/etc > /dev/null 2>&1
            print "Copying: file" $instdir/etc/$filename "."
            found="true"
         fi
      fi
   done
   if [ $found = "false" ]
   then
         print "Error: A configuration file was not found in the backup directory and is needed for successful migration."
   fi
}

function jndi_files {
   print "Backing up IBM JNDI files ....."
   mkdir $bkdir/java > /dev/null 2>&1
   mkdir $bkdir/jre > /dev/null 2>&1
   mkdir $bkdir/jre/lib > /dev/null 2>&1
   mkdir $bkdir/jre/bin > /dev/null 2>&1

   set -A jfiles
   jfiles[0]="java/ibmjcefw.jar"
   jfiles[1]="java/ibmjceprovider.jar"
   jfiles[2]="java/IBMjgssprovider.jar"
   jfiles[3]="java/Local_policy.jar"
   jfiles[4]="java/US_export_policy.jar"
   jfiles[5]="java/Krb5.ini"
   jfiles[6]="java/ibmjndi.jar"
   jfiles[7]="java/ibmjndi.zip"
   for filename in ${jfiles[*]} ;do
      if [ -f $instdir/$filename ]
      then
         cp $instdir/$filename $bkdir/java > /dev/null 2>&1
      else
         print "Skipping: file" $instdir/$filename "does not exist."
      fi
   done

   if [ -d $instdir/jre/lib ]
   then
      cp $instdir/jre/lib/* $bkdir/jre/lib > /dev/null 2>&1  
   else
      print "Skipping: directory" $instdir/jre/lib "does not exist."
   fi

   if [ -d $instdir/jre/bin ]
   then
      cp $instdir/jre/bin/* $bkdir/jre/bin > /dev/null 2>&1  
   else
      print "Skipping: directory" $instdir/jre/bin "does not exist."
   fi
}

#set -x
numparam=$#
if [ $numparam -lt 2 ]
then
   print_usage
fi

instdir=$1
bkdir=$2
jndi="no"

if [ $numparam -eq 3 ]
then
   jndi=$3
   if [ $jndi != "jndi" ]
   then
      print_usage
   fi
fi

if [ ! -d $instdir ]
then
   print "Install directory given does not exist.\n"
   print_usage
fi

if [ ! -d $bkdir ]
then
   mkdir $bkdir > /dev/null 2>&1
fi

cfg_schema_files

if [ $jndi = "jndi" ]
then
   jndi_files
fi
