#!/bin/sh
#DEBUG=TRUE

#####################################################################
#
# Header
#
# Licensed Materials - Property of IBM Corp.
# (C) Copyright IBM Corp. 2004
# 
# All Rights Reserved
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

    SCRIPT_VER=01.03
    SCRIPT_DATE=07/18/2005
    SCRIPT_SCCS=

#####################################################################
#
# History
#
# DATE        Ver    Auth         Abstract
# ----------  -----  -----------  ----------------------------------
# 11/15/2004  00.01  brookh       initial design 
# 11/18/2004  01.00  brookh       add "-v" option
# 11/26/2004  01.01  brookh       Check server status before stop/start
# 11/26/2004  01.02  brookh       .pid file is not reliable immediately after 
#                                  start/stop, so only check before first start
# 06/22/2005  01.03  dukepham     Add copying IDSWebApp.war to installableApps/SHOMEPath
#                                 and change the way this script to be executed
#                                 at <LDAPHome>/idstools/
#

#####################################################################
#
# Usage: <LDAPHome>/idstools/deploy_IDSWebApp.sh [-v]
#                                                [-w <IDSWebApp.war file>]
#                                                [-p <WAS home installed path>]
# 
# Deploys IDSWebApp.war to embeded version of Websphere Application Server Express (WAS).
# Retaining the configuration settings of the currently deployed IDSWebApp, if any.
# WAS server1 will be started.
#
# 1)  deploy_IDSWebApp.sh must executed at <LDAPHome>/idstools/.
# 
# 2)  There are 3 optional cmd line parameters which may appear in any order:
#     
#
#     -v
#
#         "version". If "-v" appears anywhere in the command line, then following
#         will be displayed:
#             -	the version and date for this deploy_IDSWebApp script 
#             -	the version and date for the new IDSWebApp.war
#             -	the version and date for the currently deployed IDSWebApp.war.
#         The new IDSWebApp.war will not be deployed.
#         Otherwise, the -w <IDSWebApp.war>, and -p <WAS home installed path>
#         parameters function as described below.
#
#
#     -w <IDSWebApp.war file>
#     
#         A fully qualified file path to the new IDSWebApp.war file to deploy.
#         Defaults to <LDAPHome>/idstools/IDSWebApp.war" if not specified.
#         After successfully deploying the new IDSWebApp.war, it will be copied
#         to "<WAS home installed path>/installableApps/". This script will rename
#         the existing IDSWebApp.war file in  "<WAS home installed path>/installableApps/"
#         to "IDSWebApp.war_bkup_mm-dd-yy_HHMMSS before the new deployed war file is
#          copied over.
#    
#
#     -p <WAS home installed path>
#
#         A fully qualified file path to the installed embedded version of 
#         WebSphere Application Server-Express (WAS).
#         Defaults to "<LDAPHome>/appsrv" if not specified.
#
#     
#####################################################################

DEBUG_FLAG="FALSE"

MAIN_PLATFORM=`eval 'uname'`
IDSWebAppWarFileName="IDSWebApp.war"
EXETIMESTAMP=`date +"%m-%d-%y_%H-%M-%S"`
DefaultWASHOMEPath=`eval 'dirname $0'`/../appsrv/
WAS_SERVER=server1
WASHOMEPath=
PATCH_SCRIPT=
WAS_TEMP=
APP_ROOT=
IDSToolsPath=
IDSWebApp=
NEW_IDSWebApp=
OPTION_V=FALSE
IDSWebAppInfo=WEB-INF/classes/IDSConfig/IDSWebAppInfo/IDSWebAppInfo.xml
CONFIG_FILES="\
    WEB-INF/classes/security/console_passwd \
    WEB-INF/classes/IDSConfig/IDSServersConfig/IDSServersInfo.xml \
    WEB-INF/classes/IDSConfig/IDSAppReg/IDSAppReg.xml \
    WEB-INF/classes/IDSConfig/IDSSessionConfig/IDSSessionMgmt.xml \
    "
NewIDSWebAppWarVer=
DeployedIDSWebAppWarVer=
QualifiedPath=

#####################################################################
#
# Set QualifiedPath of $1, or return error and clear QualifiedPath
#
QualifyPath()
{
    if [ ${DEBUG_FLAG} = TRUE ]; then
        set -x
    fi

    if [ $1 ]; then
        QualifiedPath=
        save_PWD=`pwd`
        cd `dirname ${1}` || return $?
        QualifiedPath=`pwd`/`basename ${1}`
        cd ${save_PWD}
    else                    ## $1 is null
        QualifiedPath=`pwd`
    fi
}

#####################################################################
#
# Print localized "File not found" error and return failure RC
#
AssertFileExists()
{
    if [ ${DEBUG_FLAG} = TRUE ]; then
        set -x
    fi

    if [ ! -f $1 ]; then
        ls $1 >/dev/null
    fi
}

#####################################################################
#
# Echo cmd and then call it
#
EchoCmd()
{
    if [ ${DEBUG_FLAG} = TRUE ]; then
        set -x
    fi
    
    echo ""
    echo "$*"
    eval "$*"
}

 
#####################################################################
#
# Initialize all needed variables
#
InitGlobalVariables()
{
    if [ ${DEBUG_FLAG} = TRUE ]; then
        set -x
    fi
    
    QualifyPath $1 || return $?
    IDSToolsPath=`dirname ${QualifiedPath}`
    PATCH_SCRIPT=${QualifiedPath}

    if [ ${WASHOMEPath} ]; then
       :
    else
       QualifyPath ${DefaultWASHOMEPath} || return $?
       WASHOMEPath=${QualifiedPath}
    fi

    ## Execution of setupCmdLine.sh will set all needed environment variables
    ## such as: ${WAS_HOME} and ${JAVA_HOME} etc.
    AssertFileExists "${WASHOMEPath}/bin/setupCmdLine.sh" || return $?
    . "${WASHOMEPath}/bin/setupCmdLine.sh" || return $?

    AssertFileExists "${WASHOMEPath}/bin/wsadmin.sh" || return $?

    WAS_TEMP="${WAS_HOME}/temp/deploy_IDSWebApp.tmp"
    APP_ROOT="${WAS_HOME}/installedApps/${WAS_NODE}/${IDSWebAppWarFileName?}.ear/${IDSWebAppWarFileName?}"

    if [ ${IDSWebApp} ]; then
       NEW_IDSWebApp=${IDSWebApp}
    else
       if [ "${MAIN_PLATFORM}" = "AIX" ]; then
          AssertFileExists ${IDSToolsPath}/${IDSWebAppWarFileName?} 
          if [ $? -eq 0 ]; then
             QualifyPath ${IDSToolsPath}/${IDSWebAppWarFileName?}
          else
             AssertFileExists ${IDSToolsPath}/nossl/${IDSWebAppWarFileName?} || return $?
             QualifyPath ${IDSToolsPath}/nossl/${IDSWebAppWarFileName?}
          fi
       else
          AssertFileExists ${IDSToolsPath}/${IDSWebAppWarFileName?} || return $?
          QualifyPath ${IDSToolsPath}/${IDSWebAppWarFileName?}
       fi
       NEW_IDSWebApp=${QualifiedPath}
    fi

    Verify_IDSWebApp_Filename `basename ${NEW_IDSWebApp}` || return $?
}

#####################################################################
#
# Delete ${WAS_HOME}/temp/IDSWebApp.tmp directory
#
DeleteTempDirectory()
{
    if [ ${DEBUG_FLAG} = TRUE ]; then
        set -x
    fi
    
    if [ -d ${WAS_TEMP} ]; then
        rm -rf ${WAS_TEMP}
    fi
}

#####################################################################
#
# Create empty ${WAS_HOME}/temp/IDSWebApp.tmp directory
#
CreateTempDirectory()
{
    if [ ${DEBUG_FLAG} = TRUE ]; then
        set -x
    fi
    
    DeleteTempDirectory || return $?
    mkdir -p ${WAS_TEMP} || return $?
}

#####################################################################
#
# Backup CONFIG_FILES (if any) from APP_ROOT to WAS_TEMP
#
Backup_IDSWebApp_Configuration()
{
    if [ ${DEBUG_FLAG} = TRUE ]; then
        set -x
    fi
    
    ## Set up the environment variables

    if [ ! -d ${APP_ROOT} ]; then
        return 0
    fi

    for filename in ${CONFIG_FILES}; do
        if [ -f ${APP_ROOT}/${filename} ]; then
            EchoCmd mkdir -p ${WAS_TEMP}/`dirname ${filename}` || return 1
            EchoCmd cp ${APP_ROOT}/${filename} ${WAS_TEMP}/${filename} || return 1
        fi
    done
    
    return 0
}

#####################################################################
#
# Restore CONFIG_FILES (if any) from WAS_TEMP to APP_ROOT
#
Restore_IDSWebApp_Configuration()
{
    if [ ${DEBUG_FLAG} = TRUE ]; then
        set -x
    fi
    
    for filename in ${CONFIG_FILES}; do
        if [ -f ${WAS_TEMP}/${filename} ]; then
            EchoCmd cp ${WAS_TEMP}/${filename} ${APP_ROOT}/${filename} || return 1
        fi
    done
    
    return 0
}


#####################################################################
#
# Start WAS server1
#
Start_WAS()
{
    if [ ${DEBUG_FLAG} = TRUE ]; then
        set -x
    fi
    
    EchoCmd "${WAS_HOME}/bin/startServer.sh" ${WAS_SERVER} || return $?
}

#####################################################################
#
# Stop WAS server1
#
Stop_WAS()
{
    if [ ${DEBUG_FLAG} = TRUE ]; then
        set -x
    fi

    EchoCmd "${WAS_HOME}/bin/stopServer.sh" ${WAS_SERVER} || return $?
}

#####################################################################
#
# Restart WAS server1
#
Restart_WAS()
{
    if [ ${DEBUG_FLAG} = TRUE ]; then
        set -x
    fi

    Stop_WAS
    Start_WAS || return $?
}

#####################################################################
#
# Un-deploy existing IDSWebApp.war (if found)...
#
Uninstall_IDSWebApp()
{
    if [ ${DEBUG_FLAG} = TRUE ]; then
        set -x
    fi

    if [ -d ${APP_ROOT} ]; then
        EchoCmd ${WAS_HOME}/bin/wsadmin.sh '-conntype NONE -c "\$AdminApp uninstall ${IDSWebAppWarFileName?}"' || return $?
    fi
    return 0
}

#####################################################################
#
# Deploy new IDSWebApp.war...
#
Install_IDSWebApp()
{
    if [ ${DEBUG_FLAG} = TRUE ]; then
        set -x
    fi

    EchoCmd ${WAS_HOME}/bin/wsadmin.sh '-conntype NONE -c "\$AdminApp install {'${NEW_IDSWebApp}'} {-configroot \"$CONFIG_ROOT\" -node \"$WAS_NODE\" -usedefaultbindings -nodeployejb -appname ${IDSWebAppWarFileName?} -contextroot \"IDSWebApp\"}"'
    return $?
}

#####################################################################
#
# Display app-version and build-date of IDSWebAppInfo.xml file
#   or localized 'not found' message
#
ShowIDSWebAppInfo()
{
    if [ ${DEBUG_FLAG} = TRUE ]; then
        set -x
    fi

    if [ -f $1 ]; then
        egrep '(app-version|build-date)' $1
    else
        ls $1 >/dev/null
    fi
}

#####################################################################
#
# get the major version of the new war file to:
#  ${NewIDSWebAppWarMajorVer}
#
GetVersionInfo_NewIDSWebApp()
{
    if [ ${DEBUG_FLAG} = TRUE ]; then
        set -x
    fi
    
    if [ -f ${1} ]; then
       save_PWD=`pwd`
       cd ${WAS_TEMP}
       ${JAVA_HOME}/bin/jar -xf ${1} ${IDSWebAppInfo}
       cd ${save_PWD}
    else
       return 1
    fi

    ## Get the major version of the new war file
    AssertFileExists ${WAS_TEMP}/${IDSWebAppInfo} || return $?
    temp_ver1=`egrep '(app-version)' ${WAS_TEMP}/${IDSWebAppInfo}`
    temp_ver2=`echo ${temp_ver1} | awk -F\> '{print $2}'`
    temp_ver3=`echo ${temp_ver2} | awk -F\< '{print $1}'`
    NewIDSWebAppWarMajorVer=`echo ${temp_ver3} | awk -F\. '{print $1}'` 
}

#####################################################################
#
# get the major version of deployed war in WAS:
#  ${DeployedIDSWebAppWarVer}
#
GetVersionInfo_DeployedIDSWebApp()
{
    if [ ${DEBUG_FLAG} = TRUE ]; then
        set -x
    fi
    
    if [ -f ${1} ]; then
       temp_ver1=`egrep '(app-version)' ${1}`
       temp_ver2=`echo ${temp_ver1} | awk -F\> '{print $2}'`
       temp_ver3=`echo ${temp_ver2} | awk -F\< '{print $1}'`
       DeployedIDSWebAppWarVer=`echo ${temp_ver3} | awk -F\. '{print $1}'`
    else
       return 0
    fi
}

#####################################################################
#
# "-v"
#
ShowVersionInfo()
{
    if [ ${DEBUG_FLAG} = TRUE ]; then
        set -x
    fi
    
    ## Show script version
    echo ""
    echo "${PATCH_SCRIPT}:"
    echo "    SCRIPT_VER:  ${SCRIPT_VER}"
    echo "    SCRIPT_DATE: ${SCRIPT_DATE}"
    
    ## Show version of new IDSWebApp
    echo ""
    echo "${NEW_IDSWebApp}:"
    ShowIDSWebAppInfo ${WAS_TEMP}/${IDSWebAppInfo}
}

#####################################################################
#
# Copy the new war to the <WAS install path>/installableApps/ after
#  sucessfully deployed the war file.
#
Copy_NewIDSWebApp_To_InstallableApps()
{
    if [ ${DEBUG_FLAG} = TRUE ]; then
        set -x
    fi

   ## Check if target exist if has not been checked
   AssertFileExists ${WAS_HOME}/installableApps/${IDSWebAppWarFileName?}
   if [ $? -eq 0 ]; then
      ## Backup target file before copying if it exists
      mv ${WAS_HOME}/installableApps/${IDSWebAppWarFileName?} \
         ${WAS_HOME}/installableApps/${IDSWebAppWarFileName?}_bkup_${EXETIMESTAMP} || return $?
   fi
   ## Copy to override target
   cp -Rp ${NEW_IDSWebApp} ${WAS_HOME}/installableApps/ || return $?
}

#####################################################################
#
# Only allow to deploy war file with exact name: "IDSWebApp.war"
#
Verify_IDSWebApp_Filename()
{
   if [ ${DEBUG_FLAG} = TRUE ]; then
       set -x
   fi

   ## Only set -x when about to quit
   [ "${1?}" != "${IDSWebAppWarFileName?}" ] && set -x
   [ "${1?}" != "${IDSWebAppWarFileName?}" ] && return 1
   return 0
}

#####################################################################
#
# Verify war file version and only allow equal of newer version to be
#  deployed into already deployed WAS installation.
#
Verify_IDSWebApp_Version()
{
    if [ ${DEBUG_FLAG} = TRUE ]; then
        set -x
    fi

    ## Check the war file version, and DO NOT allow back level war file
    ## to be deployed to a newer version of deployed WAS.
    if [ ${DeployedIDSWebAppWarVer} ]; then
       [ ${NewIDSWebAppWarMajorVer} -lt ${DeployedIDSWebAppWarVer} ] && set -x
       if [ ${NewIDSWebAppWarMajorVer} -lt ${DeployedIDSWebAppWarVer} ]; then
          ## Show version of new IDSWebApp
          echo ""
          echo "${NEW_IDSWebApp}:"
          ShowIDSWebAppInfo ${WAS_TEMP}/${IDSWebAppInfo}
          ## Show version of deployed IDSWebApp
          echo ""
          echo "${APP_ROOT}/${IDSWebAppInfo}:"
          ShowIDSWebAppInfo ${APP_ROOT}/${IDSWebAppInfo}
          return 1         
       fi
    fi

    return 0
}

#####################################################################
#
# main
#
main()
{
    if [ ${DEBUG_FLAG} = TRUE ]; then
        set -x
    fi

    InitGlobalVariables $1 || return $?
    CreateTempDirectory || return $?

    GetVersionInfo_NewIDSWebApp ${NEW_IDSWebApp} || return $?
    GetVersionInfo_DeployedIDSWebApp ${APP_ROOT}/${IDSWebAppInfo}

    if [ ${OPTION_V} = TRUE ]; then
        ShowVersionInfo
    else
        Verify_IDSWebApp_Version || return $?

        if [ -f ${WAS_HOME}/logs/${WAS_SERVER}/${WAS_SERVER}.pid ]; then
            Stop_WAS
            if [ $? -ne 0 ]; then
               Start_WAS || return $?
               Stop_WAS || return $?
            fi
        fi
        Backup_IDSWebApp_Configuration || return $?
        Uninstall_IDSWebApp || return $?
        Install_IDSWebApp || return $?
        Restore_IDSWebApp_Configuration || return $?
        Start_WAS || return $?
        Copy_NewIDSWebApp_To_InstallableApps || return $?
    fi
    
    DeleteTempDirectory
    echo ""
    echo "${APP_ROOT}:"
    ShowIDSWebAppInfo ${APP_ROOT}/${IDSWebAppInfo}
}

#############################################################
###################### MAIN PROGRAM #########################
#############################################################

if [ ${DEBUG} ]; then
   DEBUG_FLAG=${DEBUG}
fi

while getopts "vw:p:" opt
do
   case $opt in
   v )
      OPTION_V=TRUE
      ;;
   w )
      AssertFileExists $OPTARG || exit $?
      QualifyPath $OPTARG || exit $?
      IDSWebApp=${QualifiedPath}
      ;;
   p )
      QualifyPath $OPTARG || exit $?
      WASHOMEPath=${QualifiedPath}
      ;;
   * )
      exit $?
      ;;
   esac
done

main $0

